import Ogma, { NodeList } from '@linkurious/ogma';

const ogma = new Ogma({
  container: 'graph-container',
  renderer: 'canvas'
});

// show graph size stats
const showStats = () => {
  document.getElementById('stats')!.innerHTML =
    ogma.getNodes().size + ' nodes, ' + ogma.getEdges().size + ' edges';
};
ogma.events.on(['addNodes', 'removeNodes'], showStats);

// load graph
Ogma.parse
  .gexfFromUrl('miserables.gexf')
  .then(graph => ogma.setGraph(graph))
  .then(() => ogma.view.locateGraph());

/* Set up a keyboard shortcuts to trigger the lasso, zoom in and out, and run layout */
const toggleLasso = () => {
  if (ogma.tools.lasso.enabled()) ogma.tools.lasso.disable();
  else ogma.tools.lasso.enable();
};

ogma.events.onKeyPress('l', toggleLasso);
ogma.events.onKeyPress('space', () => {
  const selected = ogma.getSelectedNodes();
  ogma.layouts.forceLink({
    nodes: selected.size === 0 ? ogma.getNodes() : selected,
    locate: true,
    scalingRatio: 600
  });
});

ogma.events.onKeyPress('+', () => {
  ogma.view.zoomIn({ duration: 200 });
});
ogma.events.onKeyPress('-', () => {
  ogma.view.zoomOut({ duration: 200 });
});
ogma.events.onKeyPress('c', () => {
  ogma.view.locateGraph({ duration: 200 });
});

// switch rectangle select on drag start with the 'ctrl' key pressed
ogma.events.on('dragStart', () => {
  if (ogma.keyboard.isKeyPressed('ctrl')) {
    ogma.tools.rectangleSelect.enable({});
  }
});

/* Set up a handler when nodes are selected - show in the toolbar */
document.getElementById('selected-nodes')!.addEventListener('click', evt => {
  const target = evt.target as HTMLElement;
  if (/node/.test(target.className)) {
    const id = target.getAttribute('data-id');
    ogma.getNode(id).setSelected(false);
  }
});

const onSelectionChange = () => {
  const selected = ogma.getSelectedNodes();
  /* Display the number of selected node outside of Ogma */
  document.getElementById('selected-nodes-number')!.innerHTML =
    selected.size.toString();
  document.getElementById('selected-nodes')!.innerHTML = selected
    .getAttribute('text')
    .map((text, i) => {
      const node = selected.get(i);
      const id = node.getId();
      const color = node.getAttribute('color');
      return `<span class="node" style="background-color: ${color}" title="Unselect" data-id="${id}">${text}</span>`;
    })
    .join('');
};
ogma.events
  .on('nodesSelected', onSelectionChange)
  .on('removeNodes', onSelectionChange);

const hoveredContainer = document.getElementById('hovered-nodes')!;
ogma.events.on('mousemove', evt => {
  const hovered = ogma.getHoveredElement();
  if (hovered && hovered.isNode) {
    const color = hovered.getAttribute('color');
    const text = hovered.getAttribute('text');
    hoveredContainer.innerHTML = `<span class="node" style="background-color: ${color}">${text}</span>`;
  } else {
    hoveredContainer.innerHTML = '';
  }
});

// Dragging behavior
// style for the dragged nodes - red background
ogma.styles.createClass({
  name: 'dragged',
  nodeAttributes: {
    color: 'red'
  }
});
// style for the rest of the nodes - grey background
ogma.styles.createClass({
  name: 'static',
  nodeAttributes: {
    color: '#888888'
  }
});

// assign classes on drag start and remove them when the interaction is finished
ogma.events
  .on('nodesDragStart', evt => {
    // add visual style to the nodes that are not affected
    const nodes = evt.nodes;
    nodes.addClass('dragged');
    nodes.inverse().addClass('static');
  })
  .on('nodesDragEnd', evt => {
    // reset visual style for the nodes that are not affected
    const nodes = evt.nodes;
    nodes.removeClass('dragged');
    nodes.inverse().removeClass('static');
  });

// context menu routines
const menu = document.getElementById('nodes-menu')!;
let hideMenuTimer = 0;
let currentNodes: NodeList | null;
const hideMenu = () => {
  menu.classList.add('hidden');
  currentNodes = null;
  clearTimeout(hideMenuTimer);
};

menu.addEventListener('mouseover', () => {
  clearTimeout(hideMenuTimer);
});
menu.addEventListener('mouseout', () => {
  hideMenuTimer = window.setTimeout(hideMenu, 1000);
});
menu.addEventListener('click', evt => {
  const action = (evt.target as HTMLElement).getAttribute('data-action');
  if (currentNodes) {
    if (action === 'select') {
      currentNodes.setSelected(true);
    } else if (action === 'delete') {
      ogma.removeNodes(currentNodes);
    } else if (action === 'connect') {
      ogma.tools.connectNodes.enable();
    } else if (action === 'center') {
      ogma.layouts.radial({
        centralNode: currentNodes.get(0),
        radiusDelta: 300,
        repulsion: 5,
        locate: true
      });
    }
  }
  hideMenu();
});

const showMenu = (x: number, y: number, nodes: NodeList) => {
  menu.style.top = y + 'px';
  menu.style.left = x + 10 + 'px';

  clearTimeout(hideMenuTimer);

  menu.classList.remove('hidden');
  if (nodes.size === 1) {
    menu.classList.remove('group');
  } else {
    menu.classList.add('group');
  }
};

/* context menu */
ogma.events.on('click', evt => {
  const target = evt.target;
  if (evt.button === 'right' && target && target.isNode) {
    const selected = ogma.getSelectedNodes();

    // different items for menu for group or a single node
    // make sure we have a collection for consistency
    if (selected.includes(target)) {
      currentNodes = selected;
    } else {
      currentNodes = target.toList();
    }
    const position = ogma.view.graphToScreenCoordinates(target.getPosition());
    showMenu(position.x, position.y, currentNodes);
  }
});
